import importlib
import importlib_metadata

from AnyQt.QtCore import Qt
from AnyQt.QtGui import QColor, QIcon

from orangecanvas.gui.splashscreen import SplashScreen
from Orange.canvas import config, __main__ as main, mainwindow

from quasar.launcher.splash import splash_screen
from quasar.launcher.update_check import check_for_updates


def version():
    return importlib_metadata.distribution("quasar").version


class QuasarConfig(config.Config):
    ApplicationName = "Quasar"
    ApplicationVersion = version()

    @staticmethod
    def splash_screen():
        return splash_screen()

    @staticmethod
    def core_packages():
        # type: () -> List[str]
        return super(QuasarConfig, QuasarConfig).core_packages() + [
            "orange-spectroscopy",
        ]

    @staticmethod
    def application_icon():
        path = str(importlib.resources.files("quasar").joinpath("launcher/icons/quasar.svg"))
        return QIcon(path)


class QuasarSplashScreen(SplashScreen):

    def showMessage(self, message, alignment=Qt.AlignLeft, color=Qt.black):
        super().showMessage(message, alignment=alignment, color=QColor("#ff6d00"))


class Launcher:
    def launch(self):
        config.Config = QuasarConfig
        self.fix_application_dirs()
        self.replace_update_check()
        self.replace_splash_screen()
        self.main()

    def fix_application_dirs(self):
        import os, sys
        from Orange.misc import environ

        def data_dir(versioned=True):
            """
            Return the platform dependent Orange data directory.

            This is ``data_dir_base()``/Orange/__VERSION__/ directory if versioned is
            `True` and ``data_dir_base()``/Orange/ otherwise.
            """
            base = environ.data_dir_base()
            if versioned:
                return os.path.join(base, "quasar", version())
            else:
                return os.path.join(base, "quasar")

        environ.data_dir = data_dir

        def cache_dir(*args):
            """
            Return the platform dependent Orange cache directory.
            """
            if sys.platform == "darwin":
                base = os.path.expanduser("~/Library/Caches")
            elif sys.platform == "win32":
                base = os.getenv("APPDATA", os.path.expanduser("~/AppData/Local"))
            elif os.name == "posix":
                base = os.getenv("XDG_CACHE_HOME", os.path.expanduser("~/.cache"))
            else:
                base = os.path.expanduser("~/.cache")

            base = os.path.join(base, "quasar", version())
            if sys.platform == "win32":
                # On Windows cache and data dir are the same.
                # Microsoft suggest using a Cache subdirectory
                return os.path.join(base, "Cache")
            else:
                return base

        environ.cache_dir = cache_dir

    def replace_splash_screen(self):
        main.SplashScreen = QuasarSplashScreen

    def replace_update_check(self):
        from Orange.canvas import __main__
        __main__.check_for_updates = check_for_updates

    def main(self):
        from Orange.canvas.__main__ import main
        main()



import importlib

from AnyQt.QtCore import QCoreApplication, QPoint, Qt, QRect
from AnyQt.QtGui import QPixmap, QFont, QFontMetrics, QPainter, QColor


def splash_screen():
    """
    """
    pm = QPixmap(
        str(importlib.resources.files("quasar").joinpath("launcher/icons/splash.png"))
    )

    version = QCoreApplication.applicationVersion()
    size = 21 if len(version) < 6 else 16
    font = QFont("Arial")
    font.setPixelSize(size)
    metrics = QFontMetrics(font)
    br = metrics.boundingRect(version).adjusted(-5, 0, 5, 0)
    br.moveBottomLeft(QPoint(216, 82))

    p = QPainter(pm)
    p.setRenderHint(QPainter.Antialiasing)
    p.setRenderHint(QPainter.TextAntialiasing)
    p.setFont(font)
    p.setPen(QColor("#ff6d00"))
    p.drawText(br, Qt.AlignCenter, version)
    p.end()
    return pm, QRect(60, 101, 250, 328)



import logging
import time
import sys
from packaging.version import Version

from AnyQt.QtCore import QSettings, QThread, pyqtSignal, Qt, QUrl
from AnyQt.QtGui import QDesktopServices, QIcon
from AnyQt.QtWidgets import QMessageBox
from urllib.request import urlopen, Request

from orangecanvas.utils.overlay import Notification

from Orange import canvas
from Orange.version import version as current_orange
from Orange.util import resource_filename


log = logging.getLogger(__name__)


def current_version():
    from quasar.launcher import version as quasar_version
    return quasar_version()


def ua_string():
    is_anaconda = 'Continuum' in sys.version or 'conda' in sys.version
    return ('Quasar{quasar_version}:Orange{orange_version}:Python{py_version}'
            ':{platform}:{conda}:{uuid}').format(
        quasar_version=current_version(),
        orange_version=current_orange,
        py_version=sys.version.split()[0],
        platform=sys.platform,
        conda='Anaconda' if is_anaconda else '',
        uuid=QSettings().value("error-reporting/machine-id", "", str)
    )


def check_for_updates():
    settings = QSettings()
    check_updates = settings.value('startup/check-updates', True, type=bool)
    last_check_time = settings.value('startup/last-update-check-time', 0, type=int)
    ONE_DAY = 86400

    if check_updates and time.time() - last_check_time > ONE_DAY:
        settings.setValue('startup/last-update-check-time', int(time.time()))

        class GetLatestVersion(QThread):
            resultReady = pyqtSignal(str)

            def run(self):
                try:
                    request = Request('https://quasar.biolab.si/version/',
                                      headers={
                                          'Accept': 'text/plain',
                                          'Accept-Encoding': 'identity',
                                          'Connection': 'close',
                                          'User-Agent': ua_string()})
                    contents = urlopen(request, timeout=10).read().decode()
                # Nothing that this fails with should make Orange crash
                except Exception:  # pylint: disable=broad-except
                    log.exception('Failed to check for updates')
                else:
                    self.resultReady.emit(contents)

        def compare_versions(latest):
            skipped = settings.value('startup/latest-skipped-version', "", type=str)
            if Version(latest) <= Version(current_version()) or \
                    latest == skipped:
                return

            notif = Notification(title='Quasar Update Available',
                                 text='Current version: <b>{}</b><br>'
                                      'Latest version: <b>{}</b>'.format(current_version(), latest),
                                 accept_button_label="Download",
                                 reject_button_label="Skip this Version",
                                 icon=QIcon(resource_filename("canvas/icons/update.png")))

            def handle_click(role):

                if role == notif.RejectRole:
                    settings.setValue('startup/latest-skipped-version', latest)
                if role == notif.AcceptRole:
                    QDesktopServices.openUrl(QUrl("https://quasar.codes/download/"))

            notif.clicked.connect(handle_click)
            canvas.notification_server_instance.registerNotification(notif)

        thread = GetLatestVersion()
        thread.resultReady.connect(compare_versions)
        thread.start()
        return thread
    return None